# File tres/src/t_tskysub.x
# February 10, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 2009 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# TSKYSUB is an IRAF task which scales and subtracts a sky spectrum from
# an object + sky spectrum, returning an object spectrum with the same
# nonlinear dispersion function as the original object + sky spectrum
# For arguments, see parameter file tskysub.par.
# Information is shared in common blocks defined in "tresop.com".
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>

# operation defs
define OP_ADD		1
define OP_SUB		2
define OP_MUL		3
define OP_DIV		4
define OP_FUNCTIONS	"|+|-|*|/|"

procedure t_tskysub  ()

char	obsspec[SZ_PATHNAME]	# Object + sky spectrum file name
char	obspath[SZ_PATHNAME]	# Object + sky spectrum path name
char	nmobs[SZ_LINE]		# Object name
pointer pxobs			# Object + sky spectrum
pointer imobs			# Object + sky spectrum image header
pointer shobs			# Object + sky spectrum header
int	npobs			# Number of pixels in object + sky spectrum
int	nordobs			# Number of orders in object + sky spectrum

char	skyspec[SZ_PATHNAME]	# Sky spectrum file name
char	skypath[SZ_PATHNAME]	# Sky spectrum path name
char	nmsky[SZ_LINE]		# Sky spectrum name
pointer pxsky			# Sky spectrum
pointer imsky			# Sky spectrum image header
pointer shsky			# Sky spectrum header
int	npsky			# Number of pixels in sky spectrum
int	nordsky			# Number of orders in sky spectrum

char	sdir[SZ_PATHNAME]	# Directory for input spectra

char	tpspec[SZ_PATHNAME]	# Throughput correction spectrum file name
char	tppath[SZ_PATHNAME]	# Throughput correction spectrum path name
char	tpdir[SZ_PATHNAME]	# Directory for throughput correction spectrum
char	nmtp[SZ_LINE]		# Throughput correction spectrum name
pointer pxtp			# Throughput correction spectrum
pointer imtp			# Throughput correction spectrum image header
pointer shtp			# Throughput correction spectrum header
int	nptp			# Number of pixels in throughput spectrum
int	nordtp			# Number of orders in throughput spectrum

char	resspec[SZ_PATHNAME]	# Scaled sky spectrum file name
char	resdir[SZ_PATHNAME]	# Scaled sky spectrum directory
char	respath[SZ_PATHNAME]	# Scaled sky spectrum pathname
char	nmres[SZ_LINE]		# Scaled sky spectrum name
pointer imres			# Scaled sky spectrum image header structure
pointer shres			# Scaled sky spectrum header structure

char	objspec[SZ_PATHNAME]	# Object spectrum file name
char	objdir[SZ_PATHNAME]	# Object spectrum directory
char	objpath[SZ_PATHNAME]	# Object spectrum pathname
char	nmobj[SZ_LINE]		# Object spectrum name
pointer imobj			# Object spectrum image header structure
pointer shobj			# Object spectrum header structure
int	lobj

int	logfiles		# List of log files
char	logfile[SZ_PATHNAME]	# Log file name
char	tstring[SZ_LINE]
char	taskname[SZ_LINE]
char	operation[4]
int	op
int	mode
int	nlogfd
int	logfd[10]

int	i
int	order

char	str[SZ_LINE]
int	fd
int	ldir
int	npix
bool	speccur
 
bool	clgetb()
int	clpopnu(), clgeti(), clgfil(), open()
int	strlen(), strdic()
real	clgetr()

include "tresop.com"

begin
	nlogfd = 0
	imres = NULL
	shres = NULL
	imobj = NULL
	shobj = NULL
	npix = 0
	mode = READ_ONLY
	npix01 = 0
	npix02 = 0
	wl1 = NULL
	wl2 = NULL
	spix2r = NULL
	outfit = FALSE

# Get task parameters.

# Print processing information
	debug  = clgetb ("debug")

# Object plus sky spectrum
	call clgstr ("obskyspec",obsspec,SZ_PATHNAME)
	call clgstr ("specdir",sdir,SZ_PATHNAME)

# Open object + sky spectrum
	ldir = strlen (sdir)
	if (ldir > 0) {
	    if (sdir[ldir] != '/') {
		sdir[ldir+1] = '/'
		sdir[ldir+2] = EOS
		}
	    call strcpy (sdir, obspath, SZ_PATHNAME)
	    call strcat (obsspec, obspath, SZ_PATHNAME)
	    }
	else
	    call strcpy (obsspec, obspath, SZ_PATHNAME)
	call gettres (obspath, 1, pxobs, imobs, shobs, nordobs,
		      npobs, nmobs, mode, debug)
	if (debug) {
	    call printf ("TSKYSUB: First file is %s[1-%d]\n")
		call pargstr (obspath)
		call pargi (nordobs)
	    call flush (STDOUT)
	    }

# Sky spectrum
	call clgstr ("skyspec", skyspec, SZ_PATHNAME)

# Open sky spectrum
	if (ldir > 0) {
	    call strcpy (sdir, skypath, SZ_PATHNAME)
	    call strcat (skyspec, skypath, SZ_PATHNAME)
	    }
	else
	    call strcpy (skyspec, skypath, SZ_PATHNAME)
	call gettres (skypath, 1, pxsky, imsky, shsky, nordsky,
		      npsky, nmsky, mode, debug)
	IM_UPDATE(imsky) = NO
	if (debug) {
	    call printf ("TSKYSUB: Sky file is %s[1-%d]\n")
		call pargstr (skypath)
		call pargi (nordsky)
	    call flush (STDOUT)
	    }

# Print an error message and exit if not same number of orders
	if (nordobs != nordsky) {
	    call printf ("TSKYSUB: Obj+sky/Sky order mismatch %d != %d\n")
		call pargi (nordobs)
		call pargi (nordsky)
	    call flush (STDOUT)
	    return
	    }

# Sky throughput relative to object + sky throughput
	call clgstr ("tpspec", tpspec, SZ_PATHNAME)
	call clgstr ("tpdir", tpdir, SZ_PATHNAME)

# Open throughput spectrum
	ldir = strlen (tpdir)
	if (ldir > 0) {
	    if (tpdir[ldir] != '/') {
		tpdir[ldir+1] = '/'
		tpdir[ldir+2] = EOS
		}
	    call strcpy (tpdir, tppath, SZ_PATHNAME)
	    call strcat (tpspec, tppath, SZ_PATHNAME)
	    }
	else
	    call strcpy (tpspec, tppath, SZ_PATHNAME)
	call gettres (tppath, 1, pxtp, imtp, shtp, nordtp,
		      nptp, nmtp, mode, debug)
	IM_UPDATE(imtp) = NO
	if (debug) {
	    call printf ("TSKYSUB: Throughput scaling file is %s[1-%d]\n")
		call pargstr (tppath)
		call pargi (nordtp)
	    call flush (STDOUT)
	    }

# Print an error message and exit if not same number of orders
	if (nordtp != nordsky) {
	    call printf ("TSKYSUB: Throughput/Sky order mismatch %d != %d\n")
		call pargi (nordtp)
		call pargi (nordsky)
	    call flush (STDOUT)
	    return
	    }

# Plot input spectra
	pltspec = clgetb ("spec_plot")

# Interact with display of input spectra
	intspec = clgetb ("spec_int")

# Number of times to smooth (1-2-1) input data plots
	ssmooth = clgeti ("ssmooth")

# Minimum and maximum values for data in graphs
	ymin = clgetr ("ymin")
	ymax = clgetr ("ymax")

# Plot output spectrum
	pltres = clgetb ("res_plot")

# Number of times to smooth (1-2-1) output spectrum
	rsmooth = clgeti ("rsmooth")

# Interact with display of final composite spectrum
	intres = clgetb ("res_int")

# Interact with display of input spectra
	speccur = clgetb ("spec_int")

# Load fitting parameters
	call getfitpars()

# Open log files and write a header.
	logfiles = clpopnu ("logfiles")
	call fseti (STDOUT, F_FLUSHNL, YES)
	i = 0
	call strcpy ("tres.tskysub",taskname,SZ_LINE)
	while (clgfil (logfiles, logfile, SZ_PATHNAME) != EOF) {
	    fd = open (logfile, APPEND, TEXT_FILE)
	    if (fd == ERR) break
	    call loghead (taskname,str)
	    call fprintf (fd, "%s\n")
		call pargstr (str)
	    i = i + 1
	    logfd[i] = fd
	    }
	nlogfd = i
	call clpcls (logfiles)

# File to which to write scaled sky spectrum
	call clgstr ("resspec", objspec, SZ_PATHNAME)
	call clgstr ("resdir", objdir, SZ_PATHNAME)

# Set up path for output file
	lobj = strlen (objdir)
	if (lobj > 0) { 
	    if (resdir[lobj] != '/') {
		resdir[lobj+1] = '/'
		resdir[lobj+1] = EOS
		}
	    call strcpy (resdir, respath, SZ_PATHNAME)
	    call strcat (resspec, respath, SZ_PATHNAME)
	    call mkfits (respath)
	    }
	else {
	    call strcpy (resspec, respath, SZ_PATHNAME)
	    call mkfits (respath)
	    }
	call strcpy (nmsky, nmres, SZ_IMTITLE)
	call strcat (nmres, " scaled to match obj+sky throughput", SZ_IMTITLE)

#  Open the scaled sky output file with size of sky spectrum file
	call res_open (imres, respath, resspec, npsky, nordsky, imsky)
	if (imres == ERR) {
	    call eprintf ("OPSPEC: Error writing scaled sky spectrum %s\n")
		call pargstr (respath)
	    return
	    }
	else if (debug) {
	    call printf ("TSKYSUB: Scaled sky spectrum written to %s\n")
		call pargstr (respath)
	    call flush (STDOUT)
	    }
	call res_init (imsky, imres, shsky, shres, nmres)
	IM_UPDATE(imres) = YES

	if (debug) {
	    call printf ("TSKYSUB: %s initialized\n")
		call pargstr (respath)
	    call flush (STDOUT)
	    }

# Set operation to scale sky spectrum by dividing by throughput spectrum
	call strcpy ("/", operation, 4)
	op = strdic (operation,operation, 4, OP_FUNCTIONS)
	if (debug) {
	    call printf ("TSKYSUB: Divide %s by %s\n")
		call pargstr (skypath)
		call pargstr (tppath)
	    call flush (STDOUT)
	    }

# Scale sky spectrum with throughput spectrum one order at a time
	do order = 1, nordsky {
	    if (debug) {
		call printf ("TSKYSUB: %s[%d] / %s[%d]\n")
		    call pargstr (skyspec)
		    call pargi (order)
		    call pargstr (tpspec)
		    call pargi (order)
		}
	    call opspec (skyspec, skypath, tpspec, tppath, order, nordsky, op,
                  imres, respath, nmres, resspec)
	    }

# Add this operation to the ehader of the scaled sky spectrum
	call sprintf (tstring,SZ_LINE,"tres.tarith: %s %s %s")
	    call pargstr (skyspec)
	    call pargstr (operation)
	    call pargstr (tpspec)
	call imputh (imres, "HISTORY", tstring)

#  Close the output composite spectrum file
	if (imres != NULL)
	    call res_close (imres, shres, resspec, debug)

	call closetres (imsky, shsky)
	call closetres (imtp, shtp)
	if (wl1 != NULL)
	    call mfree (wl1, TY_REAL)
	if (wl2 != NULL)
	    call mfree (wl2, TY_REAL)
	if (spix2r != NULL)
	    call mfree (spix2r, TY_REAL)

# File to which to write object spectrum
	call clgstr ("objspec", objspec, SZ_PATHNAME)
	call clgstr ("objdir", objdir, SZ_PATHNAME)
	call clgstr ("objname", nmobj, SZ_PATHNAME)

# Set up path for output file
	lobj = strlen (objdir)
	if (lobj == 0) { 
	    call strcpy (objspec, objpath, SZ_PATHNAME)
	    call mkfits (objpath)
	    }
	else {
	    if (objdir[lobj] != '/') {
		objdir[lobj+1] = '/'
		objdir[lobj+1] = EOS
		}
	    call strcpy (objdir, objpath, SZ_PATHNAME)
	    call strcat (objspec, objpath, SZ_PATHNAME)
	    call mkfits (objpath)
	    }
	if (strlen (nmobj) < 1) {
	    call strcpy (nmobs, nmobj, SZ_IMTITLE)
	    }

#  Open the object output file with size of object + sky spectrum file
	call res_open (imobj, objpath, objspec, npobs, nordobs, imobs)
	if (imres == ERR) {
	    call eprintf ("OPSPEC: Error writing scaled sky spectrum %s\n")
		call pargstr (respath)
	    return
	    }
	else if (debug) {
	    call printf ("TSKYSUB: Output to %s\n")
		call pargstr (objpath)
	    }

# Subtract the scaled sky spectrum from the object + sky spectrum
	call strcpy ("-", operation, 4)
	op = strdic (operation,operation, 4, OP_FUNCTIONS)
	if (debug) {
	    call printf ("TSKYSUB: Subtract %s from %s\n")
		call pargstr (respath)
		call pargstr (obspath)
	    call flush (STDOUT)
	    }

# Subtract the scaled sky spectrum from the object + sky spectrum one order at a time
	do order = 1, nordobs {
	    if (debug) {
		call printf ("TSKYSUB: %s[%d] - %s[%d]\n")
		    call pargstr (obsspec)
		    call pargi (order)
		    call pargstr (resspec)
		    call pargi (order)
		}
	    call opspec (obsspec, obspath, resspec, respath, order, nordobs, op,
                  imobj, objpath, nmobj, objspec)
	    }

# Close the log files
	if (nlogfd > 0) {
	    do i = 1, nlogfd {
		call close (logfd[i])
		}                                              
	    }

end

# Feb 10 2009	New task based on tres/Src/t_tarith.x
