# File tres/Src/writeres.x
# January 22, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# RES_OPEN -- Open a new result file for writing

include <error.h>
include <imhdr.h>
include <imset.h>
include <fio.h>
include <mach.h>
include <fset.h>

#rebinning defs 
define RB_NEAREST	1	# nearest neighbor
define RB_LINEAR	2	# linear
define RB_POLY3		3	# 3rd order polynomial
define RB_POLY5		4	# 5th order polynomial
define RB_SPLINE3	5	# cubic spline
define RB_SINC		6	# sinc
define RB_LSINC		7	# look-up table sinc
define RB_DRIZZLE	8	# drizzle
define RB_SUMS		9	# Fractional pixel flux redistribution
define RB_FUNCTIONS	"|nearest|linear|poly3|poly5|spline3|sinc|lsinc|drizzle|sums|"


procedure res_open (im, iraffile, spectrum, npix, nspec, oldim)

pointer	im			# Output image structure (returned)
char	iraffile[ARB]		# IRAF template file name
pointer	spectrum		# Pointer to spectrum vector (retuned)
int	npix			# Number of pixels in spectrum
int	nspec			# Number of spectra in output file
pointer	oldim			# Old image structure (ignored if NULL)

int	immap()
errchk	immap
errchk	malloc

begin

#  Create IRAF image header
	if (oldim == NULL)
	    im = immap (iraffile, NEW_IMAGE, 0)
	else
	    im = immap (iraffile, NEW_COPY, oldim)
	if (im == NULL) return

# Initialize a few header parameters
	IM_PIXTYPE(im) = TY_REAL
	if (nspec < 1)
	    nspec = 1
	if (nspec > 1)
	    IM_NDIM(im) = 2
	else
	    IM_NDIM(im) = 1
	IM_LEN(im,1) = npix
	IM_PHYSLEN(im,1) = npix
	IM_LEN(im,2) = nspec
	IM_PHYSLEN(im,2) = nspec

# Allocate memory for the template spectrum
	if (spectrum != NULL)
	    call mfree (spectrum, TY_REAL)
	call malloc (spectrum,npix,TY_REAL)

	return
end


# RES_INIT initializes header of result spectrum

procedure res_init (imspec, imres, shspec, shres, resname)

pointer	imspec		# Object image header structure
pointer	imres		# Composite image header structure
pointer	shspec		# Object spectrum header structure
pointer	shres		# Composite spectrum header structure
char	resname[ARB]	# Title for output spectrum

char	dstr[SZ_LINE]	# Date string for file time stamp
char	tstring[SZ_LINE]
char	interp_mode[SZ_LINE]
int	bin_mode
char	bintype[16]
int	strlen(), clgwrd()

begin

	call clgstr ("resname",resname,SZ_IMTITLE)
	if (strlen (resname) > 0)
	    call strcpy (resname,IM_TITLE(imres),SZ_IMTITLE)
	else {
	    call strcpy (IM_TITLE(imspec),IM_TITLE(imres),SZ_IMTITLE)
	    call strcpy (IM_TITLE(imres),resname,SZ_IMTITLE)
	    }

# Get rebinning method
	bin_mode = clgwrd ("interp_mode",interp_mode,SZ_LINE,RB_FUNCTIONS)
	switch (bin_mode) {
	    case RB_NEAREST:
		call strcpy ("nearest", bintype, 16)
	    case RB_LINEAR:
		call strcpy ("linear", bintype, 16)
	    case RB_POLY3:
		call strcpy ("poly3", bintype, 16)
	    case RB_POLY5:
		call strcpy ("poly5", bintype, 16)
	    case RB_SPLINE3:
		call strcpy ("spline3", bintype, 16)
	    case RB_SINC:
		call strcpy ("sinc", bintype, 16)
	    case RB_LSINC:
		call strcpy ("lsinc", bintype, 16)
	    case RB_DRIZZLE:
		call strcpy ("drizzle", bintype, 16)
	    case RB_SUMS:
		call strcpy ("sums", bintype, 16)
	    }
	call sprintf (tstring,SZ_LINE,"tres.tarith wavelength interpolated %s")
	    call pargstr (bintype)
	call imastr (imres, "HISTORY", tstring)

	call logtime (dstr,SZ_LINE)
	call sprintf (tstring,SZ_LINE,"tres.tarith run %s")
	    call pargstr (dstr)
	call imputh (imres, "HISTORY", tstring)

# Copy spectrum header including WCS
	call shdr_copy (shspec, shres, YES)

	return
end


# RES_CLOSE -- Write pixels to result file and close it

procedure res_close (im, sh, spectrum, debug)

pointer	im		# Pointer to output image structure
pointer	sh		# Pointer to output image spectrum structure
pointer	spectrum	# Pointer to spectrum vector (retuned)
bool	debug		# True for diagnostic listing

errchk	mfree, imunmap

begin

	if (im == NULL) return

#  Move spectrum from buffer to image
	call res_write (im, 1, spectrum, TY_REAL, debug)

#  Close files and clean up.
	call closetres (im, sh)
	call mfree (spectrum,TY_REAL)

	return
end


# RES_WRITE

procedure res_write (im, ispec, tempbuf, temptype, debug)

pointer	im		# IRAF image descriptor
int	ispec		# Number of spectrum in 2D spectrum to write
pointer	tempbuf		# Template spectrum data buffer
int	temptype	# Template spectrum data type
bool	debug		# True for diagnostic output

int	npix
long	v[IM_MAXDIM], nlines
pointer	outline, templine
real	linemax, linemin
long	clktime()

errchk	res_change_pix, res_put_image_line, res_pix_lim

begin
	if (IM_NDIM(im) == 0) {
	    call printf ("RES_WRITE: No pixel file created\n")
	    return
	}

	npix = IM_LEN(im, 1)
	nlines = 1

	IM_MAX(im) = -MAX_REAL
	IM_MIN(im) = MAX_REAL

	call amovkl (long(1), v, IM_MAXDIM)
	if (ispec > 1)
	    v[2] = long (ispec)

	if (debug) {
	    call printf ("RES_WRITE:  %d pixels, type %d\n")
		call pargi (npix)
		call pargi (IM_PIXTYPE(im))
	    call flush (STDOUT)
	    }

# Set output image line buffer
	templine = tempbuf
	call res_put_image_line (im, outline, v, IM_PIXTYPE(im))

# Change pixels to appropriate type
	call res_change_pix (templine, outline, npix, temptype, IM_PIXTYPE(im))

# Calculate image maximum and minimum
	call res_pix_lim (outline, npix, IM_PIXTYPE(im), linemin, linemax)
	IM_MAX(im) = max (IM_MAX(im), linemax)
	IM_MIN(im) = min (IM_MIN(im), linemin)
	IM_CTIME(im) = clktime (long(0))

	if (debug) {
	    call printf ("RES_WRITE:  minimum is %f, maximum is %f\n")
		call pargr (IM_MIN(im))
		call pargr (IM_MAX(im))
	    call flush (STDOUT)
	    }
	return
end


# RES_PUT_IMAGE_LINE -- Get a buffer pointer to output a line to an IRAF file

procedure res_put_image_line (im, buf, v, data_type)

pointer	im			# IRAF image descriptor
pointer	buf			# Pointer to output image line
long	v[ARB]			# imio pointer
int	data_type		# output pixel type

int	impnll(), impnlr(), impnld(), impnlx(), impnls()
errchk	impnll, impnlr, impnld, impnlx, impnls

begin
	switch (data_type) {
	case TY_SHORT, TY_USHORT:
	    if (impnls (im, buf, v) == EOF)
		call error (3, "RES_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_INT, TY_LONG:
	    if (impnll (im, buf, v) == EOF)
		call error (3, "RES_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_REAL:
	    if (impnlr (im, buf, v) == EOF)
		call error (3, "RES_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_DOUBLE:
	    if (impnld (im, buf, v) == EOF)
		call error (3, "RES_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_COMPLEX:
	    if (impnlx (im, buf, v) == EOF)
		call error (3, "RES_PUT_IMAGE_LINE: Error writing IRAF file")
	default:
	    call error (10, "RES_PUT_IMAGE_LINE: Unsupported IRAF image type")
	}
	return
end


# RES_CHANGE_PIX -- Change a line of numbers to the IRAF image type

procedure res_change_pix (inbuf, outbuf, npix, in_type, out_type)

pointer inbuf			# array of archive data
pointer	outbuf			# pointer to IRAF image line
int	npix			# number of pixels
int	in_type			# input pixel type
int	out_type		# output pixel type

begin
	switch (out_type) {
	    case TY_SHORT, TY_USHORT:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtss (Mems[inbuf], Mems[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtls (Memi[inbuf], Mems[outbuf], npix)
		    case TY_REAL:
			Call achtrs (Memr[inbuf], Mems[outbuf], npix)
		    case TY_DOUBLE:
			Call achtds (Memd[inbuf], Mems[outbuf], npix)
		    default:
			call error (10, "RES_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_INT, TY_LONG:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsl (Mems[inbuf], Meml[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtll (Memi[inbuf], Meml[outbuf], npix)
		    case TY_REAL:
			Call achtrl (Memr[inbuf], Meml[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdl (Memd[inbuf], Meml[outbuf], npix)
		    default:
			call error (10, "RES_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_REAL:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsr (Mems[inbuf], Memr[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtlr (Memi[inbuf], Memr[outbuf], npix)
		    case TY_REAL:
			Call achtrr (Memr[inbuf], Memr[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdr (Memd[inbuf], Memr[outbuf], npix)
		    default:
			call error (10, "RES_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_DOUBLE:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsd (Mems[inbuf], Meml[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtld (Memi[inbuf], Meml[outbuf], npix)
		    case TY_REAL:
			Call achtrd (Memr[inbuf], Meml[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdd (Memd[inbuf], Meml[outbuf], npix)
		    default:
			call error (10, "RES_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_COMPLEX:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsx (Mems[inbuf], Memx[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtlx (Memi[inbuf], Memx[outbuf], npix)
		    case TY_REAL:
			Call achtrx (Memr[inbuf], Memx[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdx (Memd[inbuf], Memx[outbuf], npix)
		    default:
			call error (10, "RES_CHANGE_LINE: Illegal archive type")
		    }
	    default:
		call error (10, "RES_CHANGE_LINE: Illegal IRAF image type")
	    }
	return
end


# RES_PIX_LIMITS -- Determine the maximum and minimum values in a line

procedure res_pix_lim (buf, npix, pixtype, linemin, linemax)

pointer	buf			# pointer to IRAF image line
int	npix			# number of pixels
int	pixtype			# output data type
real	linemax, linemin	# min and max pixel values

short	smax, smin
long	lmax, lmin
real	rmax, rmin
double	dmax, dmin
complex	xmax, xmin

begin
	switch (pixtype) {
	case TY_SHORT, TY_USHORT:
	    call alims (Mems[buf], npix, smin, smax)
	    linemax = smax
	    linemin = smin
	case TY_INT, TY_LONG:
	    call aliml (Meml[buf], npix, lmin, lmax)
	    linemax = lmax
	    linemin = lmin
	case TY_REAL:
	    call alimr (Memr[buf], npix, rmin, rmax)
	    linemax = rmax
	    linemin = rmin
	case TY_DOUBLE:
	    call alimd (Memd[buf], npix, dmin, dmax)
	    linemax = dmax
	    linemin = dmin
	case TY_COMPLEX:
	    call alimx (Memx[buf], npix, xmin, xmax)
	    linemax = xmax
	    linemin = xmin
	default:
	    call error (30, "RES_PIX_LIMITS: Unknown IRAF type")
	}
	return
end


# MKFITS -- Change filename to explicit FITS file

procedure mkfits (filename)

char	filename[ARB]	# File name to fix

int	iext, strsearch()

begin
	iext = strsearch (filename, ".imh")
	if (iext > 0) {
	    filename[iext-4] = EOS
	    call strcat (".fits", filename, SZ_PATHNAME)
	    return
	    }
	iext = strsearch (filename, ".fit")
	if (iext == 0)
	    call strcat (".fits", filename, SZ_PATHNAME)

	return
end

# Jan 22 2009	New subroutines based on rvsao/Util/writetemp.x
